@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2017-2021 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -40,7 +40,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
 	memcpy(in, data, size);
 	in[size] = 0;
 
-	wget_http_response_t *resp = wget_http_parse_response_header(in);
+	wget_http_response *resp = wget_http_parse_response_header(in);
 	wget_http_free_response(&resp);
 
 	free(in);
