@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -43,21 +43,19 @@
  * Routines for URL extraction from Atom feeds.
  */
 
-struct _atom_context {
-	wget_vector_t
+struct atom_context {
+	wget_vector
 		*urls;
 };
 
-static void _atom_get_url(void *context, int flags, const char *dir, const char *attr, const char *val, size_t len, size_t pos G_GNUC_WGET_UNUSED)
+static void atom_get_url(void *context, int flags, const char *dir, const char *attr, const char *val, size_t len, size_t pos WGET_GCC_UNUSED)
 {
-	struct _atom_context *ctx = context;
-	wget_string_t url;
+	struct atom_context *ctx = context;
+	wget_string *url;
 
 	if (!val || !len)
 		return;
 
-	url.p = NULL;
-
 	if ((flags & XML_FLG_ATTRIBUTE)) {
 		if (!wget_strcasecmp_ascii(attr, "href") || !wget_strcasecmp_ascii(attr, "uri")
 			|| !wget_strcasecmp_ascii(attr, "src") || !wget_strcasecmp_ascii(attr, "scheme")
@@ -66,13 +64,16 @@ static void _atom_get_url(void *context, int flags, const char *dir, const char
 			for (;len && c_isspace(*val); val++, len--); // skip leading spaces
 			for (;len && c_isspace(val[len - 1]); len--);  // skip trailing spaces
 
-			url.p = val;
-			url.len = len;
+			if (!(url = wget_malloc(sizeof(wget_string))))
+				return;
+
+			url->p = val;
+			url->len = len;
 
 			if (!ctx->urls)
 				ctx->urls = wget_vector_create(32, NULL);
 
-			wget_vector_add(ctx->urls, &url, sizeof(url));
+			wget_vector_add(ctx->urls, url);
 		}
 	}
 	else if ((flags & XML_FLG_CONTENT)) {
@@ -89,13 +90,16 @@ static void _atom_get_url(void *context, int flags, const char *dir, const char
 
 				// debug_printf("#2 %02X %s %s '%.*s' %zd\n", flags, dir, attr, (int) len, val, len);
 
-				url.p = val;
-				url.len = len;
+				if (!(url = wget_malloc(sizeof(wget_string))))
+					return;
+
+				url->p = val;
+				url->len = len;
 
 				if (!ctx->urls)
 					ctx->urls = wget_vector_create(32, NULL);
 
-				wget_vector_add(ctx->urls, &url, sizeof(url));
+				wget_vector_add(ctx->urls, url);
 			}
 		}
 	}
@@ -109,11 +113,11 @@ static void _atom_get_url(void *context, int flags, const char *dir, const char
  * as vector in \p urls.
  *
  */
-void wget_atom_get_urls_inline(const char *atom, wget_vector_t **urls)
+void wget_atom_get_urls_inline(const char *atom, wget_vector **urls)
 {
-	struct _atom_context context = { .urls = NULL };
+	struct atom_context context = { .urls = NULL };
 
-	wget_xml_parse_buffer(atom, _atom_get_url, &context, XML_HINT_REMOVE_EMPTY_CONTENT);
+	wget_xml_parse_buffer(atom, atom_get_url, &context, XML_HINT_REMOVE_EMPTY_CONTENT);
 
 	*urls = context.urls;
 }
