@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2014 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2014 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -46,18 +46,16 @@
  * This is an implementation of RFC 2560.
  */
 
-typedef struct {
-	wget_ocsp_db_t
-		parent;
-	char *
+struct wget_ocsp_db_st {
+	const char *
 		fname;
-	wget_hashmap_t *
+	wget_hashmap *
 		fingerprints;
-	wget_hashmap_t *
+	wget_hashmap *
 		hosts;
-	wget_thread_mutex_t
+	wget_thread_mutex
 		mutex;
-} _ocsp_db_impl_t;
+};
 
 typedef struct {
 	const char *
@@ -68,12 +66,22 @@ typedef struct {
 		mtime; // creation time
 	bool
 		valid : 1; // 1=valid, 0=revoked
-} _ocsp_t;
+} ocsp_entry;
+
+/// Pointer to the function table
+static const wget_ocsp_db_vtable
+	*plugin_vtable;
+
+void wget_ocsp_set_plugin(const wget_ocsp_db_vtable *vtable)
+{
+	plugin_vtable = vtable;
+}
 
 #ifdef __clang__
 __attribute__((no_sanitize("integer")))
 #endif
-static unsigned int G_GNUC_WGET_PURE _hash_ocsp(const _ocsp_t *ocsp)
+WGET_GCC_PURE
+static unsigned int hash_ocsp(const ocsp_entry *ocsp)
 {
 	unsigned int hash = 0;
 	const unsigned char *p;
@@ -84,44 +92,55 @@ static unsigned int G_GNUC_WGET_PURE _hash_ocsp(const _ocsp_t *ocsp)
 	return hash;
 }
 
-static int G_GNUC_WGET_NONNULL_ALL G_GNUC_WGET_PURE _compare_ocsp(const _ocsp_t *h1, const _ocsp_t *h2)
+WGET_GCC_NONNULL_ALL WGET_GCC_PURE
+static int compare_ocsp(const ocsp_entry *h1, const ocsp_entry *h2)
 {
 	return strcmp(h1->key, h2->key);
 }
 
-static _ocsp_t *_init_ocsp(_ocsp_t *ocsp)
+static ocsp_entry *init_ocsp(ocsp_entry *ocsp)
 {
-	if (!ocsp)
-		ocsp = xmalloc(sizeof(_ocsp_t));
+	if (!ocsp) {
+		if (!(ocsp = wget_calloc(1, sizeof(ocsp_entry))))
+			return NULL;
+	} else
+		memset(ocsp, 0, sizeof(*ocsp));
 
-	memset(ocsp, 0, sizeof(*ocsp));
 	ocsp->mtime = time(NULL);
 
 	return ocsp;
 }
 
-static void _deinit_ocsp(_ocsp_t *ocsp)
+static void deinit_ocsp(ocsp_entry *ocsp)
 {
 	if (ocsp) {
 		xfree(ocsp->key);
 	}
 }
 
-static void _free_ocsp(_ocsp_t *ocsp)
+static void free_ocsp(ocsp_entry *ocsp)
 {
 	if (ocsp) {
-		_deinit_ocsp(ocsp);
+		deinit_ocsp(ocsp);
 		xfree(ocsp);
 	}
 }
 
-static _ocsp_t *_new_ocsp(const char *fingerprint, time_t maxage, int valid)
+static ocsp_entry *new_ocsp(const char *fingerprint, int64_t maxage, bool valid)
 {
-	_ocsp_t *ocsp = _init_ocsp(NULL);
+	if (fingerprint) {
+		if (!(fingerprint = wget_strdup(fingerprint)))
+			return NULL;
+	}
 
-	ocsp->key = wget_strdup(fingerprint);
-	ocsp->maxage = maxage;
-	ocsp->valid = !!valid;
+	ocsp_entry *ocsp = init_ocsp(NULL);
+
+	if (ocsp) {
+		ocsp->key = fingerprint;
+		ocsp->maxage = maxage;
+		ocsp->valid = valid;
+	} else
+		xfree(fingerprint);
 
 	return ocsp;
 }
@@ -131,7 +150,7 @@ static _ocsp_t *_new_ocsp(const char *fingerprint, time_t maxage, int valid)
  * \param[in] fingerprint The public key fingerprint to search for
  * \param[out] revoked If the key is found, the value will be set to 1 if the key has been revoked,
  *                     zero if not. If the key is not found, the value is unmodified.
- * \return 1 if the fingerprint was found, 0 otherwise.
+ * \return %true if the fingerprint was found, %false otherwise.
  *
  * Searches for a cached OCSP response in the OCSP database. OCSP responses are added using
  * wget_ocsp_db_add_fingerprint().
@@ -141,28 +160,25 @@ static _ocsp_t *_new_ocsp(const char *fingerprint, time_t maxage, int valid)
  * This function is thread-safe and can be called from multiple threads concurrently.
  * Any implementation for this function must be thread-safe as well.
  */
-int wget_ocsp_fingerprint_in_cache(const wget_ocsp_db_t *ocsp_db, const char *fingerprint, int *revoked)
+bool wget_ocsp_fingerprint_in_cache(const wget_ocsp_db *ocsp_db, const char *fingerprint, int *revoked)
 {
-	if (ocsp_db)
-		return ocsp_db->vtable->fingerprint_in_cache(ocsp_db, fingerprint, revoked);
+	if (plugin_vtable)
+		return plugin_vtable->fingerprint_in_cache(ocsp_db, fingerprint, revoked);
 
-	return 0;
-}
-static bool impl_ocsp_db_fingerprint_in_cache(const wget_ocsp_db_t *ocsp_db, const char *fingerprint, int *revoked)
-{
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
+	if (!ocsp_db)
+		return false;
 
-	_ocsp_t ocsp, *ocspp;
+	ocsp_entry ocsp, *ocspp;
 
 	// look for an exact match
 	ocsp.key = fingerprint;
-	if ((ocspp = wget_hashmap_get(ocsp_db_priv->fingerprints, &ocsp)) && ocspp->maxage >= (int64_t) time(NULL)) {
+	if (wget_hashmap_get(ocsp_db->fingerprints, &ocsp, &ocspp) && ocspp->maxage >= (int64_t) time(NULL)) {
 		if (revoked)
 			*revoked = !ocspp->valid;
-		return 1;
+		return true;
 	}
 
-	return 0;
+	return false;
 }
 
 /**
@@ -179,26 +195,23 @@ static bool impl_ocsp_db_fingerprint_in_cache(const wget_ocsp_db_t *ocsp_db, con
  *
  * \see wget_ocsp_db_add_host
  */
-bool wget_ocsp_hostname_is_valid(const wget_ocsp_db_t *ocsp_db, const char *hostname)
+bool wget_ocsp_hostname_is_valid(const wget_ocsp_db *ocsp_db, const char *hostname)
 {
-	if (ocsp_db)
-		return ocsp_db->vtable->hostname_is_valid(ocsp_db, hostname);
+	if (plugin_vtable)
+		return plugin_vtable->hostname_is_valid(ocsp_db, hostname);
 
-	return 0;
-}
-static bool impl_ocsp_db_hostname_is_valid(const wget_ocsp_db_t *ocsp_db, const char *hostname)
-{
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
+	if (!ocsp_db)
+		return false;
 
-	_ocsp_t ocsp, *ocspp;
+	ocsp_entry ocsp, *ocspp;
 
 	// look for an exact match
 	ocsp.key = hostname;
-	if ((ocspp = wget_hashmap_get(ocsp_db_priv->hosts, &ocsp)) && ocspp->maxage >= (int64_t) time(NULL)) {
-		return 1;
+	if (wget_hashmap_get(ocsp_db->hosts, &ocsp, &ocspp) && ocspp->maxage >= (int64_t) time(NULL)) {
+		return true;
 	}
 
-	return 0;
+	return false;
 }
 
 /**
@@ -210,18 +223,21 @@ static bool impl_ocsp_db_hostname_is_valid(const wget_ocsp_db_t *ocsp_db, const
  *
  * If `ocsp_db` is NULL then this function does nothing.
  */
-void wget_ocsp_db_deinit(wget_ocsp_db_t *ocsp_db)
+void wget_ocsp_db_deinit(wget_ocsp_db *ocsp_db)
 {
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
+	if (plugin_vtable) {
+		plugin_vtable->deinit(ocsp_db);
+		return;
+	}
 
-	if (ocsp_db_priv) {
-		xfree(ocsp_db_priv->fname);
-		wget_thread_mutex_lock(ocsp_db_priv->mutex);
-		wget_hashmap_free(&ocsp_db_priv->fingerprints);
-		wget_hashmap_free(&ocsp_db_priv->hosts);
-		wget_thread_mutex_unlock(ocsp_db_priv->mutex);
+	if (ocsp_db) {
+		xfree(ocsp_db->fname);
+		wget_thread_mutex_lock(ocsp_db->mutex);
+		wget_hashmap_free(&ocsp_db->fingerprints);
+		wget_hashmap_free(&ocsp_db->hosts);
+		wget_thread_mutex_unlock(ocsp_db->mutex);
 
-		wget_thread_mutex_destroy(&ocsp_db_priv->mutex);
+		wget_thread_mutex_destroy(&ocsp_db->mutex);
 	}
 }
 
@@ -233,63 +249,60 @@ void wget_ocsp_db_deinit(wget_ocsp_db_t *ocsp_db)
  * A double pointer is required because this function will set the handle (pointer) to the HPKP database to NULL
  * to prevent potential use-after-free conditions.
  *
- * New entries added to the database will be lost unless commited to the persistent storage using
+ * New entries added to the database will be lost unless committed to the persistent storage using
  * wget_ocsp_db_save().
  *
  * If `ocsp_db` or the pointer it points to is NULL, then this function does nothing.
  */
-void wget_ocsp_db_free(wget_ocsp_db_t **ocsp_db)
+void wget_ocsp_db_free(wget_ocsp_db **ocsp_db)
 {
-	if (! ocsp_db || ! *ocsp_db)
+	if (plugin_vtable) {
+		plugin_vtable->free(ocsp_db);
 		return;
+	}
 
-	(*ocsp_db)->vtable->free(*ocsp_db);
-	*ocsp_db = NULL;
-}
-static void impl_ocsp_db_free(wget_ocsp_db_t *ocsp_db)
-{
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
-
-	wget_ocsp_db_deinit((wget_ocsp_db_t *) ocsp_db_priv);
-	xfree(ocsp_db_priv);
+	if (ocsp_db && *ocsp_db) {
+		wget_ocsp_db_deinit(*ocsp_db);
+		xfree(*ocsp_db);
+	}
 }
 
-static void _ocsp_db_add_fingerprint_entry(_ocsp_db_impl_t *ocsp_db_priv, _ocsp_t *ocsp)
+static void ocsp_db_add_fingerprint_entry(wget_ocsp_db *ocsp_db, ocsp_entry *ocsp)
 {
 	if (!ocsp)
 		return;
 
-	if (!ocsp_db_priv) {
-		_free_ocsp(ocsp);
+	if (!ocsp_db) {
+		free_ocsp(ocsp);
 		return;
 	}
 
-	wget_thread_mutex_lock(ocsp_db_priv->mutex);
+	wget_thread_mutex_lock(ocsp_db->mutex);
 
 	if (ocsp->maxage == 0) {
-		if (wget_hashmap_remove(ocsp_db_priv->fingerprints, ocsp))
+		if (wget_hashmap_remove(ocsp_db->fingerprints, ocsp))
 			debug_printf("removed OCSP cert %s\n", ocsp->key);
-		_free_ocsp(ocsp);
+		free_ocsp(ocsp);
 	} else {
-		_ocsp_t *old = wget_hashmap_get(ocsp_db_priv->fingerprints, ocsp);
+		ocsp_entry *old;
 
-		if (old) {
+		if (wget_hashmap_get(ocsp_db->fingerprints, ocsp, &old)) {
 			if (old->mtime < ocsp->mtime) {
 				old->mtime = ocsp->mtime;
 				old->maxage = ocsp->maxage;
 				old->valid = ocsp->valid;
 				debug_printf("update OCSP cert %s (maxage=%lld,valid=%d)\n", old->key, (long long)old->maxage, old->valid);
 			}
-			_free_ocsp(ocsp);
+			free_ocsp(ocsp);
 		} else {
 			// key and value are the same to make wget_hashmap_get() return old 'ocsp'
 			debug_printf("add OCSP cert %s (maxage=%lld,valid=%d)\n", ocsp->key, (long long)ocsp->maxage, ocsp->valid);
-			wget_hashmap_put_noalloc(ocsp_db_priv->fingerprints, ocsp, ocsp);
+			wget_hashmap_put(ocsp_db->fingerprints, ocsp, ocsp);
 			// no need to free anything here
 		}
 	}
 
-	wget_thread_mutex_unlock(ocsp_db_priv->mutex);
+	wget_thread_mutex_unlock(ocsp_db->mutex);
 }
 
 /**
@@ -307,56 +320,54 @@ static void _ocsp_db_add_fingerprint_entry(_ocsp_db_impl_t *ocsp_db_priv, _ocsp_
  * This function is thread-safe and can be called from multiple threads concurrently.
  * Any implementation for this function must be thread-safe as well.
  */
-void wget_ocsp_db_add_fingerprint(wget_ocsp_db_t *ocsp_db, const char *fingerprint, time_t maxage, int valid)
+void wget_ocsp_db_add_fingerprint(wget_ocsp_db *ocsp_db, const char *fingerprint, int64_t maxage, bool valid)
 {
-	if (ocsp_db)
-		ocsp_db->vtable->add_fingerprint(ocsp_db, fingerprint, maxage, valid);
-}
-static void impl_ocsp_db_add_fingerprint(wget_ocsp_db_t *ocsp_db, const char *fingerprint, time_t maxage, int valid)
-{
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
+	if (plugin_vtable) {
+		plugin_vtable->add_fingerprint(ocsp_db, fingerprint, maxage, valid);
+		return;
+	}
 
-	_ocsp_t *ocsp = _new_ocsp(fingerprint, maxage, valid);
+	ocsp_entry *ocsp = new_ocsp(fingerprint, maxage, valid);
 
-	_ocsp_db_add_fingerprint_entry(ocsp_db_priv, ocsp);
+	ocsp_db_add_fingerprint_entry(ocsp_db, ocsp);
 }
 
-static void _ocsp_db_add_host_entry(_ocsp_db_impl_t *ocsp_db_priv, _ocsp_t *ocsp)
+static void ocsp_db_add_host_entry(wget_ocsp_db *ocsp_db, ocsp_entry *ocsp)
 {
 	if (!ocsp)
 		return;
 
-	if (!ocsp_db_priv) {
-		_free_ocsp(ocsp);
+	if (!ocsp_db) {
+		free_ocsp(ocsp);
 		return;
 	}
 
-	wget_thread_mutex_lock(ocsp_db_priv->mutex);
+	wget_thread_mutex_lock(ocsp_db->mutex);
 
 	if (ocsp->maxage == 0) {
-		if (wget_hashmap_remove(ocsp_db_priv->hosts, ocsp))
+		if (wget_hashmap_remove(ocsp_db->hosts, ocsp))
 			debug_printf("removed OCSP host %s\n", ocsp->key);
-		_free_ocsp(ocsp);
+		free_ocsp(ocsp);
 	} else {
-		_ocsp_t *old = wget_hashmap_get(ocsp_db_priv->hosts, ocsp);
+		ocsp_entry *old;
 
-		if (old) {
+		if (wget_hashmap_get(ocsp_db->hosts, ocsp, &old)) {
 			if (old->mtime < ocsp->mtime) {
 				old->mtime = ocsp->mtime;
 				old->maxage = ocsp->maxage;
 				old->valid = ocsp->valid;
 				debug_printf("update OCSP host %s (maxage=%lld)\n", old->key, (long long)old->maxage);
 			}
-			_free_ocsp(ocsp);
+			free_ocsp(ocsp);
 		} else {
 			// key and value are the same to make wget_hashmap_get() return old 'ocsp'
-			wget_hashmap_put_noalloc(ocsp_db_priv->hosts, ocsp, ocsp);
+			wget_hashmap_put(ocsp_db->hosts, ocsp, ocsp);
 			debug_printf("add OCSP host %s (maxage=%lld)\n", ocsp->key, (long long)ocsp->maxage);
 			// no need to free anything here
 		}
 	}
 
-	wget_thread_mutex_unlock(ocsp_db_priv->mutex);
+	wget_thread_mutex_unlock(ocsp_db->mutex);
 }
 
 /**
@@ -378,26 +389,24 @@ static void _ocsp_db_add_host_entry(_ocsp_db_impl_t *ocsp_db_priv, _ocsp_t *ocsp
  * This function is thread-safe and can be called from multiple threads concurrently.
  * Any implementation for this function must be thread-safe as well.
  */
-void wget_ocsp_db_add_host(wget_ocsp_db_t *ocsp_db, const char *host, time_t maxage)
+void wget_ocsp_db_add_host(wget_ocsp_db *ocsp_db, const char *host, int64_t maxage)
 {
-	if (ocsp_db)
-		ocsp_db->vtable->add_host(ocsp_db, host, maxage);
-}
-static void impl_ocsp_db_add_host(wget_ocsp_db_t *ocsp_db, const char *host, time_t maxage)
-{
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
+	if (plugin_vtable) {
+		plugin_vtable->add_host(ocsp_db, host, maxage);
+		return;
+	}
 
-	_ocsp_t *ocsp = _new_ocsp(host, maxage, 0);
+	ocsp_entry *ocsp = new_ocsp(host, maxage, false);
 
-	_ocsp_db_add_host_entry(ocsp_db_priv, ocsp);
+	ocsp_db_add_host_entry(ocsp_db, ocsp);
 }
 
 // load the OCSP cache from a flat file
 // not thread-save
 
-static int _ocsp_db_load(_ocsp_db_impl_t *ocsp_db_priv, FILE *fp, int load_hosts)
+static int ocsp_db_load(wget_ocsp_db *ocsp_db, FILE *fp, bool load_hosts)
 {
-	_ocsp_t ocsp;
+	ocsp_entry ocsp;
 	char *buf = NULL, *linep, *p;
 	size_t bufsize = 0;
 	ssize_t buflen;
@@ -417,7 +426,7 @@ static int _ocsp_db_load(_ocsp_db_impl_t *ocsp_db_priv, FILE *fp, int load_hosts
 		while (buflen > 0 && (buf[buflen] == '\n' || buf[buflen] == '\r'))
 			buf[--buflen] = 0;
 
-		_init_ocsp(&ocsp);
+		init_ocsp(&ocsp);
 		ok = 0;
 
 		// parse cert's sha-256 checksum
@@ -432,7 +441,7 @@ static int _ocsp_db_load(_ocsp_db_impl_t *ocsp_db_priv, FILE *fp, int load_hosts
 			ocsp.maxage = (int64_t) atoll(p);
 			if (ocsp.maxage < now) {
 				// drop expired entry
-				_deinit_ocsp(&ocsp);
+				deinit_ocsp(&ocsp);
 				continue;
 			}
 			ok = 1;
@@ -447,16 +456,16 @@ static int _ocsp_db_load(_ocsp_db_impl_t *ocsp_db_priv, FILE *fp, int load_hosts
 		// parse mtime (age of this entry)
 		if (*linep) {
 			for (p = ++linep; *linep && !isspace(*linep);) linep++;
-			ocsp.valid = !!atoi(p);
+			ocsp.valid = atoi(p) != 0;
 		}
 
 		if (ok) {
 			if (load_hosts)
-				_ocsp_db_add_host_entry(ocsp_db_priv, wget_memdup(&ocsp, sizeof(ocsp)));
+				ocsp_db_add_host_entry(ocsp_db, wget_memdup(&ocsp, sizeof(ocsp)));
 			else
-				_ocsp_db_add_fingerprint_entry(ocsp_db_priv, wget_memdup(&ocsp, sizeof(ocsp)));
+				ocsp_db_add_fingerprint_entry(ocsp_db, wget_memdup(&ocsp, sizeof(ocsp)));
 		} else {
-			_deinit_ocsp(&ocsp);
+			deinit_ocsp(&ocsp);
 			error_printf(_("Failed to parse OCSP line: '%s'\n"), buf);
 		}
 	}
@@ -469,14 +478,14 @@ static int _ocsp_db_load(_ocsp_db_impl_t *ocsp_db_priv, FILE *fp, int load_hosts
 	return 0;
 }
 
-static int _ocsp_db_load_hosts(void *ocsp_db_priv, FILE *fp)
+static int ocsp_db_load_hosts(void *ocsp_db, FILE *fp)
 {
-	return _ocsp_db_load(ocsp_db_priv, fp, 1);
+	return ocsp_db_load(ocsp_db, fp, true);
 }
 
-static int _ocsp_db_load_fingerprints(void *ocsp_db_priv, FILE *fp)
+static int ocsp_db_load_fingerprints(void *ocsp_db, FILE *fp)
 {
-	return _ocsp_db_load(ocsp_db_priv, fp, 0);
+	return ocsp_db_load(ocsp_db, fp, false);
 }
 
 /**
@@ -490,60 +499,56 @@ static int _ocsp_db_load_fingerprints(void *ocsp_db_priv, FILE *fp)
  *
  * If `ocsp_db` is NULL then this function returns -1 and does nothing else.
  */
-int wget_ocsp_db_load(wget_ocsp_db_t *ocsp_db)
+int wget_ocsp_db_load(wget_ocsp_db *ocsp_db)
 {
-	if (ocsp_db)
-		return ocsp_db->vtable->load(ocsp_db);
-
-	return -1;
-}
-static int impl_ocsp_db_load(wget_ocsp_db_t *ocsp_db)
-{
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
+	if (plugin_vtable)
+		return plugin_vtable->load(ocsp_db);
 
 	int ret;
 
-	if (!ocsp_db_priv->fname || !*ocsp_db_priv->fname)
+	if (!ocsp_db->fname || !*ocsp_db->fname)
 		return -1;
 
-	char fname_hosts[strlen(ocsp_db_priv->fname) + 6 + 1];
-	wget_snprintf(fname_hosts, sizeof(fname_hosts), "%s_hosts", ocsp_db_priv->fname);
+	char fname_hosts[strlen(ocsp_db->fname) + 6 + 1];
+	wget_snprintf(fname_hosts, sizeof(fname_hosts), "%s_hosts", ocsp_db->fname);
 
-	if ((ret = wget_update_file(fname_hosts, _ocsp_db_load_hosts, NULL, ocsp_db_priv)))
+	if ((ret = wget_update_file(fname_hosts, ocsp_db_load_hosts, NULL, ocsp_db)))
 		error_printf(_("Failed to read OCSP hosts\n"));
 	else
-		debug_printf(_("Fetched OCSP hosts from '%s'\n"), fname_hosts);
+		debug_printf("Fetched OCSP hosts from '%s'\n", fname_hosts);
 
-	if (wget_update_file(ocsp_db_priv->fname, _ocsp_db_load_fingerprints, NULL, ocsp_db_priv)) {
+	if (wget_update_file(ocsp_db->fname, ocsp_db_load_fingerprints, NULL, ocsp_db)) {
 		error_printf(_("Failed to read OCSP fingerprints\n"));
 		ret = -1;
 	} else
-		debug_printf(_("Fetched OCSP fingerprints from '%s'\n"), ocsp_db_priv->fname);
+		debug_printf("Fetched OCSP fingerprints from '%s'\n", ocsp_db->fname);
 
 	return ret;
 }
 
-static int G_GNUC_WGET_NONNULL_ALL _ocsp_save_fingerprint(FILE *fp, const _ocsp_t *ocsp)
+WGET_GCC_NONNULL_ALL
+static int ocsp_save_fingerprint(FILE *fp, const ocsp_entry *ocsp)
 {
 	wget_fprintf(fp, "%s %lld %lld %d\n", ocsp->key, (long long)ocsp->maxage, (long long)ocsp->mtime, ocsp->valid);
 	return 0;
 }
 
-static int G_GNUC_WGET_NONNULL_ALL _ocsp_save_host(FILE *fp, const _ocsp_t *ocsp)
+WGET_GCC_NONNULL_ALL
+static int ocsp_save_host(FILE *fp, const ocsp_entry *ocsp)
 {
 	wget_fprintf(fp, "%s %lld %lld\n", ocsp->key, (long long)ocsp->maxage, (long long)ocsp->mtime);
 	return 0;
 }
 
-static int _ocsp_db_save_hosts(void *ocsp_db_priv, FILE *fp)
+static int ocsp_db_save_hosts(void *ocsp_db, FILE *fp)
 {
-	wget_hashmap_t *map = ((_ocsp_db_impl_t *)ocsp_db_priv)->hosts;
+	wget_hashmap *map = ((wget_ocsp_db *) ocsp_db)->hosts;
 
 	if ((wget_hashmap_size(map)) > 0) {
 		fputs("#OCSP 1.0 host file\n", fp);
-		fputs("#Generated by Wget " PACKAGE_VERSION ". Edit at your own risk.\n", fp);
+		fputs("#Generated by libwget " PACKAGE_VERSION ". Edit at your own risk.\n", fp);
 		fputs("<hostname> <time_t maxage> <time_t mtime>\n\n", fp);
-		wget_hashmap_browse(map, (wget_hashmap_browse_t)_ocsp_save_host, fp);
+		wget_hashmap_browse(map, (wget_hashmap_browse_fn *) ocsp_save_host, fp);
 
 		if (ferror(fp))
 			return -1;
@@ -552,16 +557,16 @@ static int _ocsp_db_save_hosts(void *ocsp_db_priv, FILE *fp)
 	return 0;
 }
 
-static int _ocsp_db_save_fingerprints(void *ocsp_db_priv, FILE *fp)
+static int ocsp_db_save_fingerprints(void *ocsp_db, FILE *fp)
 {
-	wget_hashmap_t *map = ((_ocsp_db_impl_t *)ocsp_db_priv)->fingerprints;
+	wget_hashmap *map = ((wget_ocsp_db *) ocsp_db)->fingerprints;
 
 	if ((wget_hashmap_size(map)) > 0) {
 
 		fputs("#OCSP 1.0 fingerprint file\n", fp);
 		fputs("#Generated by Wget " PACKAGE_VERSION ". Edit at your own risk.\n", fp);
 		fputs("<sha256 fingerprint of cert> <time_t maxage> <time_t mtime> <valid>\n\n", fp);
-		wget_hashmap_browse(map, (wget_hashmap_browse_t)_ocsp_save_fingerprint, fp);
+		wget_hashmap_browse(map, (wget_hashmap_browse_fn *) ocsp_save_fingerprint, fp);
 
 		if (ferror(fp))
 			return -1;
@@ -581,51 +586,33 @@ static int _ocsp_db_save_fingerprints(void *ocsp_db_priv, FILE *fp)
  *
  * If `ocsp_db` is NULL then this function returns -1 and does nothing else.
  */
-int wget_ocsp_db_save(wget_ocsp_db_t *ocsp_db)
+int wget_ocsp_db_save(wget_ocsp_db *ocsp_db)
 {
-	if (ocsp_db)
-		return ocsp_db->vtable->save(ocsp_db);
-
-	return -1;
-}
-// Save the OCSP hosts and fingerprints to flat files.
-// Protected by flock()
-static int impl_ocsp_db_save(wget_ocsp_db_t *ocsp_db)
-{
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
+	if (plugin_vtable)
+		return plugin_vtable->save(ocsp_db);
 
 	int ret;
 
-	if (!ocsp_db_priv || !ocsp_db_priv->fname || !*ocsp_db_priv->fname)
+	if (!ocsp_db || !ocsp_db->fname || !*ocsp_db->fname)
 		return -1;
 
-	char fname_hosts[strlen(ocsp_db_priv->fname) + 6 + 1];
-	wget_snprintf(fname_hosts, sizeof(fname_hosts), "%s_hosts", ocsp_db_priv->fname);
+	char fname_hosts[strlen(ocsp_db->fname) + 6 + 1];
+	wget_snprintf(fname_hosts, sizeof(fname_hosts), "%s_hosts", ocsp_db->fname);
 
-	if ((ret = wget_update_file(fname_hosts, _ocsp_db_load_hosts, _ocsp_db_save_hosts, ocsp_db_priv)))
+	if ((ret = wget_update_file(fname_hosts, ocsp_db_load_hosts, ocsp_db_save_hosts, ocsp_db)))
 		error_printf(_("Failed to write to OCSP hosts to '%s'\n"), fname_hosts);
 	else
-		debug_printf(_("Saved OCSP hosts to '%s'\n"), fname_hosts);
+		debug_printf("Saved OCSP hosts to '%s'\n", fname_hosts);
 
-	if (wget_update_file(ocsp_db_priv->fname, _ocsp_db_load_fingerprints, _ocsp_db_save_fingerprints, ocsp_db_priv)) {
-		error_printf(_("Failed to write to OCSP fingerprints to '%s'\n"), ocsp_db_priv->fname);
+	if (wget_update_file(ocsp_db->fname, ocsp_db_load_fingerprints, ocsp_db_save_fingerprints, ocsp_db)) {
+		error_printf(_("Failed to write to OCSP fingerprints to '%s'\n"), ocsp_db->fname);
 		ret = -1;
 	} else
-		debug_printf(_("Saved OCSP fingerprints to '%s'\n"), ocsp_db_priv->fname);
+		debug_printf("Saved OCSP fingerprints to '%s'\n", ocsp_db->fname);
 
 	return ret;
 }
 
-static struct wget_ocsp_db_vtable vtable = {
-	.load = impl_ocsp_db_load,
-	.save = impl_ocsp_db_save,
-	.fingerprint_in_cache = impl_ocsp_db_fingerprint_in_cache,
-	.hostname_is_valid = impl_ocsp_db_hostname_is_valid,
-	.add_fingerprint = impl_ocsp_db_add_fingerprint,
-	.add_host = impl_ocsp_db_add_host,
-	.free = impl_ocsp_db_free
-};
-
 /**
  * \param[in] ocsp_db OCSP database handle previously passed to wget_ocsp_db_deinit(), or NULL
  * \param[in] fname The filename from where OCSP entries should be loaded, or NULL
@@ -635,29 +622,45 @@ static struct wget_ocsp_db_vtable vtable = {
  *
  * This function does no file IO, OCSP entries are read from `fname` into memory when wget_ocsp_db_load() is called.
  */
-wget_ocsp_db_t *wget_ocsp_db_init(wget_ocsp_db_t *ocsp_db, const char *fname)
+wget_ocsp_db *wget_ocsp_db_init(wget_ocsp_db *ocsp_db, const char *fname)
 {
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
+	if (plugin_vtable)
+		return plugin_vtable->init(ocsp_db, fname);
 
-	if (!ocsp_db_priv)
-		ocsp_db_priv = xmalloc(sizeof(_ocsp_db_impl_t));
-
-	memset(ocsp_db_priv, 0, sizeof(*ocsp_db_priv));
-
-	ocsp_db_priv->parent.vtable = &vtable;
 	if (fname)
-		ocsp_db_priv->fname = wget_strdup(fname);
-	ocsp_db_priv->fingerprints = wget_hashmap_create(16, (wget_hashmap_hash_t)_hash_ocsp, (wget_hashmap_compare_t)_compare_ocsp);
-	wget_hashmap_set_key_destructor(ocsp_db_priv->fingerprints, (wget_hashmap_key_destructor_t)_free_ocsp);
-	wget_hashmap_set_value_destructor(ocsp_db_priv->fingerprints, (wget_hashmap_value_destructor_t)_free_ocsp);
+		if (!(fname = wget_strdup(fname)))
+			return NULL;
 
-	ocsp_db_priv->hosts = wget_hashmap_create(16, (wget_hashmap_hash_t)_hash_ocsp, (wget_hashmap_compare_t)_compare_ocsp);
-	wget_hashmap_set_key_destructor(ocsp_db_priv->hosts, (wget_hashmap_key_destructor_t)_free_ocsp);
-	wget_hashmap_set_value_destructor(ocsp_db_priv->hosts, (wget_hashmap_value_destructor_t)_free_ocsp);
+	wget_hashmap *fingerprints = wget_hashmap_create(16, (wget_hashmap_hash_fn *) hash_ocsp, (wget_hashmap_compare_fn *) compare_ocsp);
+	wget_hashmap *hosts = wget_hashmap_create(16, (wget_hashmap_hash_fn *) hash_ocsp, (wget_hashmap_compare_fn *) compare_ocsp);
 
-	wget_thread_mutex_init(&ocsp_db_priv->mutex);
+	if (!fingerprints || !hosts) {
+no_mem:
+		wget_hashmap_free(&hosts);
+		wget_hashmap_free(&fingerprints);
+		xfree(fname);
+		return NULL;
+	}
 
-	return (wget_ocsp_db_t *) ocsp_db_priv;
+	if (!ocsp_db) {
+		if (!(ocsp_db = wget_calloc(1, sizeof(struct wget_ocsp_db_st))))
+			goto no_mem;
+	} else
+		memset(ocsp_db, 0, sizeof(*ocsp_db));
+
+	ocsp_db->fname = fname;
+
+	wget_hashmap_set_key_destructor(fingerprints, (wget_hashmap_key_destructor *) free_ocsp);
+	wget_hashmap_set_value_destructor(fingerprints, (wget_hashmap_value_destructor *) free_ocsp);
+	ocsp_db->fingerprints = fingerprints;
+
+	wget_hashmap_set_key_destructor(hosts, (wget_hashmap_key_destructor *) free_ocsp);
+	wget_hashmap_set_value_destructor(hosts, (wget_hashmap_value_destructor *) free_ocsp);
+	ocsp_db->hosts = hosts;
+
+	wget_thread_mutex_init(&ocsp_db->mutex);
+
+	return (wget_ocsp_db *) ocsp_db;
 }
 
 /**
@@ -667,13 +670,10 @@ wget_ocsp_db_t *wget_ocsp_db_init(wget_ocsp_db_t *ocsp_db, const char *fname)
  * Changes the file from where OCSP database entries would be loaded or saved.
  * Works only with OCSP databases created with wget_ocsp_db_init().
  */
-void wget_ocsp_db_set_fname(wget_ocsp_db_t *ocsp_db, const char *fname)
+void wget_ocsp_db_set_fname(wget_ocsp_db *ocsp_db, const char *fname)
 {
-	_ocsp_db_impl_t *ocsp_db_priv = (_ocsp_db_impl_t *) ocsp_db;
-
-	xfree(ocsp_db_priv->fname);
-	if (fname)
-		ocsp_db_priv->fname = wget_strdup(fname);
+	xfree(ocsp_db->fname);
+	ocsp_db->fname = wget_strdup(fname);
 }
 
 /**@}*/
