@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -103,7 +103,7 @@ static void _free_host_entry(HOST *host)
 	}
 }
 
-HOST *host_add(wget_iri *iri)
+HOST *host_add(const wget_iri *iri)
 {
 	wget_thread_mutex_lock(hosts_mutex);
 
@@ -125,7 +125,7 @@ HOST *host_add(wget_iri *iri)
 	return hostp;
 }
 
-HOST *host_get(wget_iri *iri)
+HOST *host_get(const wget_iri *iri)
 {
 	HOST *hostp, host = { .scheme = iri->scheme, .host = iri->host, .port = iri->port };
 
@@ -161,6 +161,14 @@ static int _search_queue_for_free_job(struct _find_free_job_context *ctx, JOB *j
 			}
 		}
 	} else if (!job->inuse) {
+		// job may be paused due to a failure (retry later)
+		long long pause = job->retry_ts - ctx->now;
+		if (pause > 0) {
+			if (!ctx->pause || ctx->pause < pause)
+				ctx->pause = pause;
+			return 0;
+		}
+
 		job->inuse = job->done = 1;
 		job->used_by = wget_thread_self();
 		job->part = NULL;
@@ -206,7 +214,7 @@ static int WGET_GCC_NONNULL_ALL _search_host_for_free_job(struct _find_free_job_
 	// find next job to do
 	wget_list_browse(host->queue, (wget_list_browse_fn *) _search_queue_for_free_job, ctx);
 
-	return !!ctx->job; // 1=found a job, 0=no free job
+	return ctx->job != 0; // 1=found a job, 0=no free job
 }
 
 /**
@@ -249,7 +257,7 @@ static int _release_job(wget_thread_id *ctx, JOB *job)
 			if (part->inuse && part->used_by == self) {
 				part->inuse = 0;
 				part->used_by = 0;
-				debug_printf("released chunk %d/%d %s\n", it + 1, wget_vector_size(job->parts), job->local_filename);
+				debug_printf("released chunk %d/%d %s\n", it + 1, wget_vector_size(job->parts), job->blacklist_entry->local_filename);
 			}
 		}
 	} else if (job->inuse && job->used_by == self) {
@@ -295,7 +303,8 @@ void host_add_job(HOST *host, const JOB *job)
 {
 	JOB *jobp;
 
-	debug_printf("%s: job fname %s\n", __func__, job->local_filename);
+	if (job->blacklist_entry)
+		debug_printf("%s: job fname %s\n", __func__, job->blacklist_entry->local_filename);
 
 	wget_thread_mutex_lock(hosts_mutex);
 
@@ -323,14 +332,20 @@ void host_add_job(HOST *host, const JOB *job)
  * This function creates a priority job for robots.txt.
  * This job has to be processed before any other job.
  */
-void host_add_robotstxt_job(HOST *host, wget_iri *iri, bool http_fallback)
+void host_add_robotstxt_job(HOST *host, const wget_iri *base, const char *encoding, bool http_fallback)
 {
 	JOB *job;
+	blacklist_entry *blacklist_robots;
+	wget_iri *robot_iri = wget_iri_parse_base(base, "/robots.txt", encoding);
 
-	job = job_init(NULL, iri, http_fallback);
+	if (!robot_iri || !(blacklist_robots = blacklist_add(robot_iri))) {
+		wget_iri_free(&robot_iri);
+		return;
+	}
+
+	job = job_init(NULL, blacklist_robots, http_fallback);
 	job->host = host;
 	job->robotstxt = 1;
-	job->local_filename = get_local_filename(job->iri);
 
 	wget_thread_mutex_lock(hosts_mutex);
 	host->robot_job = job;
@@ -490,7 +505,7 @@ void host_queue_free(HOST *host)
 /*
 static int _queue_print_func(void *context WGET_GCC_UNUSED, JOB *job)
 {
-	debug_printf("  %s %d\n", job->local_filename, job->inuse);
+	debug_printf("  %s %d\n", job->blacklist_entry->local_filename, job->inuse);
 	return 0;
 }
 
