@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2014 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2014 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -30,20 +30,20 @@
 
 bool bar_init(void);
 void bar_deinit(void);
-void bar_print(int slot, const char *s) G_GNUC_WGET_NONNULL_ALL;
-void bar_printf(int slot, const char *fmt, ...) G_GNUC_WGET_PRINTF_FORMAT(2,3) G_GNUC_WGET_NONNULL_ALL;
-void bar_vprintf(int slot, const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(2,0) G_GNUC_WGET_NONNULL_ALL;
-void bar_slot_begin(int slot, const char *filename, ssize_t filesize);
+void bar_print(int slot, const char *s) WGET_GCC_NONNULL_ALL;
+void bar_printf(int slot, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3) WGET_GCC_NONNULL_ALL;
+void bar_vprintf(int slot, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0) WGET_GCC_NONNULL_ALL;
+void bar_slot_begin(int slot, const char *filename, int new_file, ssize_t filesize);
 void bar_set_downloaded(int slot, size_t nbytes);
 void bar_slot_deregister(int slot);
 void bar_update_slots(int nslots);
 
 /*
 ssize_t
-	wget_bar_vprintf(wget_bar_t *bar, int slot, const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(3,0) G_GNUC_WGET_NONNULL_ALL;
+	wget_bar_vprintf(wget_bar *bar, int slot, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(3,0) WGET_GCC_NONNULL_ALL;
 ssize_t
-	wget_bar_printf(wget_bar_t *bar, int slot, const char *fmt, ...) G_GNUC_WGET_PRINTF_FORMAT(3,4) G_GNUC_WGET_NONNULL_ALL;
+	wget_bar_printf(wget_bar *bar, int slot, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4) WGET_GCC_NONNULL_ALL;
 void
-	wget_bar_print(wget_bar_t *bar, int slot, const char *s)G_GNUC_WGET_NONNULL_ALL;
+	wget_bar_print(wget_bar *bar, int slot, const char *s) WGET_GCC_NONNULL_ALL;
 */
 #endif /* SRC_WGET_BAR_H */
