@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2021 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -31,22 +31,51 @@
 #include <stdarg.h>
 #include <unistd.h> // needed for EXIT_SUCCESS
 
+#ifdef WITH_LIBHSTS
+#include <libhsts.h>
+#endif
+
 #include <wget.h>
 
+#define INCLUDED_DIRECTORY_PREFIX '+'
+#define EXCLUDED_DIRECTORY_PREFIX '-'
+
 //types for --https-enforce
-enum {
-	WGET_HTTPS_ENFORCE_NONE,
-	WGET_HTTPS_ENFORCE_SOFT,
-	WGET_HTTPS_ENFORCE_HARD
-};
+typedef enum {
+	HTTPS_ENFORCE_NONE,
+	HTTPS_ENFORCE_SOFT,
+	HTTPS_ENFORCE_HARD
+} https_enforce_mode;
+
+typedef enum {
+	GPG_VERIFY_DISABLED,
+	GPG_VERIFY_SIG_FAIL,
+	WGET_GPG_VERIFY_SIG_NO_FAIL
+} gpg_verify_mode;
+
+typedef enum {
+	DOWNLOAD_ATTR_NO,
+	DOWNLOAD_ATTR_STRIPPATH,
+	DOWNLOAD_ATTR_USEPATH,
+} download_attr_mode;
+
+typedef struct {
+	const char
+		*filename;
+	FILE
+		*fp;
+	wget_stats_format
+		format;
+} stats_args;
 
 struct config {
-	wget_iri_t
+	wget_iri
 		*base;
 	const char
 		*post_file,
 		*post_data,
-		*gnutls_options,
+		*body_file,
+		*body_data,
 		*http_username,
 		*http_password,
 		*http_proxy_username,
@@ -55,6 +84,7 @@ struct config {
 		*local_encoding,  // encoding of the environment and file system
 		*remote_encoding, // encoding of remote files (if not specified in Content-Type HTTP header or in document itself)
 		*bind_address,
+		*bind_interface,
 		*input_file,
 		*base_url,
 		*default_page,
@@ -82,15 +112,23 @@ struct config {
 		*reject_regex,
 		*gnupg_homedir,
 		*stats_all,
-		*stats_dns,
-		*stats_ocsp,
-		*stats_server,
-		*stats_site,
-		*stats_tls;
-	wget_vector_t
+		*system_config,
+		*user_config,
+		*hsts_file,
+		*hsts_preload_file,
+		*hpkp_file,
+		*tls_session_file,
+		*ocsp_server,
+		*ocsp_file,
+		*netrc_file,
+		*use_askpass_bin,
+		*hostname,
+		*dns_cache_preload,
+		*method;
+	wget_vector
 		*compression,
-		*config_files,
 		*domains,
+		*exclude_directories,
 		*exclude_domains,
 		*accept_patterns,
 		*reject_patterns,
@@ -101,38 +139,42 @@ struct config {
 		*ignore_tags,
 		*default_challenges,
 		*headers,
-		*mime_types;
-	wget_content_encoding_type_t
+		*mime_types,
+		*retry_on_http_error,
+		*save_content_on;
+	wget_content_encoding
 		compression_methods[wget_content_encoding_max + 1];	// the last one for counting
-	wget_hsts_db_t
+	wget_hsts_db
 		*hsts_db; // in-memory HSTS database
-	wget_hpkp_db_t
+#ifdef WITH_LIBHSTS
+	hsts_t
+		*hsts_preload_data; // in-memory HSTS preloaded data
+#endif
+	wget_hpkp_db
 		*hpkp_db; // in-memory HPKP database
-	wget_tls_session_db_t
+	wget_tls_session_db
 		*tls_session_db; // in-memory TLS session database
-	wget_ocsp_db_t
+	wget_ocsp_db
 		*ocsp_db; // in-memory fingerprint OCSP database
-	wget_netrc_db_t
+	wget_netrc_db
 		*netrc_db; // in-memory .netrc database
-	struct _wget_cookie_db_st
+	wget_cookie_db
 		*cookie_db;
+	stats_args
+		*stats_dns_args,
+		*stats_ocsp_args,
+		*stats_server_args,
+		*stats_site_args,
+		*stats_tls_args;
 	char
-		*hsts_file,
-		*hpkp_file,
-		*tls_session_file,
-		*ocsp_file,
-		*netrc_file,
 		*password,
-		*use_askpass_bin,
 		*username;
 	size_t
 		chunk_size;
 	long long
-		quota;
-	bool
-		no_compression,
-		auth_no_challenge,
-		verify_sig;
+		quota,
+		limit_rate, // bytes
+		start_pos; // bytes
 	int
 		http2_request_window,
 		backups,
@@ -148,24 +190,38 @@ struct config {
 		read_timeout, // ms
 		max_redirect,
 		max_threads;
-	unsigned short
+	uint16_t
 		default_http_port,
 		default_https_port;
+	wget_report_speed
+		report_speed;
+	https_enforce_mode
+		https_enforce;
+	gpg_verify_mode
+		verify_sig;
 	char
-		tls_resume,            // if TLS session resumption is enabled or not
-		tls_false_start,
+		cert_type,             // SSL_X509_FMT_PEM or SSL_X509_FMT_DER (=ASN1)
+		private_key_type,      // SSL_X509_FMT_PEM or SSL_X509_FMT_DER (=ASN1)
 		progress,
+		regex_type,
+		download_attr;
+	bool
+		tls_resume,            // if TLS session resumption is enabled or not
 		content_on_error,
 		fsync_policy,
 		netrc,
 		http2,
+		http2_only,
 		ocsp_stapling,
 		ocsp,
 		mirror,
 		backup_converted,
+		convert_file_only,
 		convert_links,
 		ignore_case,
+		ignore_length,
 		hsts,                  // if HSTS (HTTP Strict Transport Security) is enabled or not
+		hsts_preload,          // if loading of a HSTS Preload file is enabled of not
 		hpkp,                  // HTTP Public Key Pinning (HPKP)
 		random_wait,
 		trust_server_names,
@@ -197,21 +253,18 @@ struct config {
 		continue_download,
 		server_response,
 		keep_alive,
+		keep_extension,
 		keep_session_cookies,
 		cookies,
 		spider,
 		dns_caching,
-		tcp_fastopen,
 		check_certificate,
 		check_hostname,
-		cert_type,             // SSL_X509_FMT_PEM or SSL_X509_FMT_DER (=ASN1)
-		private_key_type,      // SSL_X509_FMT_PEM or SSL_X509_FMT_DER (=ASN1)
 		span_hosts,
-		recursive,
 		verbose,
-		print_version,
 		quiet,
 		debug,
+		hyperlink,
 		metalink,
 		cut_url_get_vars,
 		cut_file_get_vars,
@@ -220,41 +273,48 @@ struct config {
 		force_progress,
 		local_db,
 		dont_write, // fuzzers and unit/fuzz tests set this to 1, so they won't write any files
-		regex_type,
 		filter_urls,
 		askpass,
-		report_speed,
 		verify_save_failed,
-		https_enforce,
-		retry_connrefused;
+		retry_connrefused,
+		unlink,
+		background,
+		if_modified_since,
+		auth_no_challenge,
+		no_compression,
+		ocsp_date,
+		ocsp_nonce,
+		recursive,
+		tls_false_start,
+		tcp_fastopen;
 };
 
 extern struct config
 	config;
 
-typedef enum exit_status_t {
-	WG_EXIT_STATUS_NO_ERROR       = EXIT_SUCCESS,
-	WG_EXIT_STATUS_GENERIC        = 1,
-	WG_EXIT_STATUS_PARSE_INIT     = 2,
-	WG_EXIT_STATUS_IO             = 3,
-	WG_EXIT_STATUS_NETWORK        = 4,
-	WG_EXIT_STATUS_TLS            = 5,
-	WG_EXIT_STATUS_AUTH           = 6,
-	WG_EXIT_STATUS_PROTOCOL       = 7,
-	WG_EXIT_STATUS_REMOTE         = 8,
-	WG_EXIT_STATUS_GPG_ERROR      = 9
-} exit_status_t;
+typedef enum {
+	EXIT_STATUS_NO_ERROR       = EXIT_SUCCESS,
+	EXIT_STATUS_GENERIC        = 1,
+	EXIT_STATUS_PARSE_INIT     = 2,
+	EXIT_STATUS_IO             = 3,
+	EXIT_STATUS_NETWORK        = 4,
+	EXIT_STATUS_TLS            = 5,
+	EXIT_STATUS_AUTH           = 6,
+	EXIT_STATUS_PROTOCOL       = 7,
+	EXIT_STATUS_REMOTE         = 8,
+	EXIT_STATUS_GPG_ERROR      = 9
+} exit_status_e;
 
 // Needed for fuzzers that are compiled by C++
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-int init(int argc, const char **argv) G_GNUC_WGET_NONNULL_ALL;
+int init(int argc, const char **argv) WGET_GCC_NONNULL_ALL;
 int selftest_options(void);
 void deinit(void);
-void set_exit_status(exit_status_t status);
-exit_status_t get_exit_status(void);
+void set_exit_status(exit_status_e status);
+exit_status_e get_exit_status(void);
 
 #ifdef __cplusplus
 }
